/*MIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDI

                           P L A Y U M I D I
                  (/dev/umixiX.Y Direct MIDI Player)

   Copyright(C) 2014 Koine Yuusuke(koinec). All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

 1. Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY Koine Yuusuke(koinec) ``AS IS'' AND ANY
EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL Koine Yuusuke(koinec) OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
OF THE POSSIBILITY OF SUCH DAMAGE.

MIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDIMIDI*/


#define SMF_HEADER_STRING 		"MThd"
#define SMF_TRKHEADER_STRING    "MTrk"

#define	SMF_EVENT_NOTEOFF		0x80		/* 3Byte */
#define	SMF_EVENT_NOTEON		0x90		/* 3Byte */
#define	SMF_EVENT_KEYPRESSER	0xa0		/* 3Byte */
#define	SMF_EVENT_CTRLCHANGE	0xb0		/* 3Byte */
#define	SMF_EVENT_PROGCHANGE	0xc0		/* 2Byte */
#define	SMF_EVENT_CHNPRESSER	0xd0		/* 2Byte */
#define	SMF_EVENT_PITCHBEND		0xe0		/* 3Byte */
#define	SMF_EVENT_SYSEX_META	0xf0		/* Variable Size */

#define	SMF_EVENT_SYSEX_F0		0xf0
#define	SMF_EVENT_SYSEX_F7		0xf7
#define	SMF_EVENT_META			0xff

#define	SMF_META_SEQUENCENO		0x00		/* 3Byte=0x02(len), 4,5Byte=Data */
#define	SMF_META_TEXT			0x01		/* 3Byte=len, 4Byte- =Data */
#define	SMF_META_COPYRIGHT		0x02		/* 3Byte=len, 4Byte- =Data */
#define	SMF_META_TITLE			0x03		/* 3Byte=len, 4Byte- =Data */
#define	SMF_META_INSTRUMENT		0x04		/* 3Byte=len, 4Byte- =Data */
#define	SMF_META_LYLIC			0x05		/* 3Byte=len, 4Byte- =Data */
#define	SMF_META_MARKER			0x06		/* 3Byte=len, 4Byte- =Data */
#define	SMF_META_QUEPOINT		0x07		/* 3Byte=len, 4Byte- =Data */
#define	SMF_META_PROGRAM		0x08		/* 3Byte=len, 4Byte- =Data */
#define	SMF_META_DEVICE			0x09		/* 3Byte=len, 4Byte- =Data */
#define	SMF_META_CHNPREFIX		0x20		/* 3Byte=0x01(len), 4Byte=Data */
#define	SMF_META_PORT			0x21		/* 3Byte=0x01(len), 4Byte=Data(Port) */
#define	SMF_META_TRACKEND		0x2f		/* 3Byte=0x00(len) */
#define	SMF_META_TEMPO			0x51		/* 3Byte=0x03(len), 4,5,6Byte=TempoData */
#define	SMF_META_SMPTEOFFSET	0x54		/* 3Byte=0x05(len), 4Byte- = hr,mn,se,fr,ff */
#define	SMF_META_TIMESIGNATURE	0x58		/* 3Byte=0x04(len), 4Byte- = nn,dd,cc,bb */
#define	SMF_META_KEYSIGNATURE	0x59		/* 3Byte=0x02(len), 4Byte- = sf,mi */
#define	SMF_META_SPECIAL		0x7f		/* 3Byte=len, 4Byte- =Data */

#define	SMF_IS_RUNNING_STATUS(n)	(!((n)&0x80))

#define	MIDI_VENDOR_SEQUENTIAL		0x01	/* Group America */
#define	MIDI_VENDOR_CLARITY			0x1f
#define	MIDI_VENDOR_PASSAC			0x20	/* Group Europe */
#define	MIDI_VENDOR_WERSI			0x3b
#define	MIDI_VENDOR_KAWAI			0x40	/* Group Japan */
#define	MIDI_VENDOR_ROLAND			0x41
#define	MIDI_VENDOR_KORG			0x42
#define	MIDI_VENDOR_YAMAHA			0x43
#define	MIDI_VENDOR_CASIO			0x44
#define	MIDI_VENDOR_KAMIYA_STUDIO	0x46
#define	MIDI_VENDOR_AKAI			0x47
#define	MIDI_VENDOR_JAPAN_VICTOR	0x48
#define	MIDI_VENDOR_MEISOSHA		0x49
#define	MIDI_VENDOR_HOSHINO_GAKKI	0x4a
#define	MIDI_VENDOR_FUJITSU			0x4b
#define	MIDI_VENDOR_SONY			0x4c
#define	MIDI_VENDOR_NISSIN_ONPA		0x4d
#define	MIDI_VENDOR_TEAC			0x4e
#define	MIDI_VENDOR_SYSTEM_PRODUCT	0x4f
#define	MIDI_VENDOR_MATUSHITA		0x50
#define	MIDI_VENDOR_FOSTEX			0x51

#define	MIDI_RESETSIZE_GM			6
#define	MIDI_RESETSIZE_XG			9
#define	MIDI_RESETSIZE_GS			11

#define	MIDI_RESET_GM	{ 0xf0, 0x7e, 0x7f, 0x09, 0x01, 0xf7 }
#define	MIDI_RESET_XG	\
		{ 0xf0, 0x43, 0x10 /*dev*/, 0x4c, 0x00, 0x00, \
			0x7e, 0x00, 0xf7 }
#define	MIDI_RESET_GS	\
		{ 0xf0, 0x41, 0x20 /*dev*/, 0x42, 0x12, 0x40, \
			0x00, 0x7f, 0x00, 0x41, 0xf7 }


/* EOF of smf.h *********************************************************/
